@@ -65,6 +65,7 @@ particularly integer conversions.
 #include <functional>
 #include <string>
 #include <cassert>
+#include <memory>
 
 #if INT_MAX != 0x7fffffff
   #error "Unexpected value for INT_MAX"
@@ -103,7 +104,7 @@ class PlExceptionBase : public std::exception
 };
 
 
-// PlFail is a pseudo-exception for quick exist on failure, for use by
+// PlFail is a pseudo-exception for quick exit on failure, for use by
 // the PlTerm::unify methods and PlQuery::next_solution().  This is
 // special-cased in the PREDICATE et al macros.  Note that it is *not*
 // a subclass of PlException. See the documentation for more details
@@ -122,7 +123,7 @@ public:
 // PlExceptionFail is a variant of PlFail, for when a resource error
 // happens and we can't use PlException (because we're out of resources
 // and therefore can't create any more terms).
-class PlExceptionFail : public PlExceptionBase 
+class PlExceptionFail : public PlExceptionBase
 {
 public:
   explicit PlExceptionFail() {}
@@ -139,7 +140,7 @@ public:
 // wish to wrap the call in PlCheckFail().
 template<typename C_t> [[nodiscard]] C_t PlWrap(C_t rc, qid_t qid = 0);
 
-// As PlWrap, but always throw an exception for non-zero rc.
+// As PlWrap, but always throw an exception for zero rc.
 // This is for functions that report errors but don't have an
 // indication of "fail" - that is, almost everything except for
 // functions like PL_unify_*() or PL_next_solution().
@@ -180,12 +181,13 @@ public:
   bool operator ==(const WrappedC<C_t>& o) const { return C_ == o.C_; }
   bool operator !=(const WrappedC<C_t>& o) const { return C_ != o.C_; }
 
-  // reset() is common with "smart pointers"; wrapped wrapped atom_t,
-  // term_t, etc. aren't "smart" in the same sense, but the objects
-  // they refer to are garbage collected and some care is needed to
+  // reset() is common with "smart pointers"; wrapped atom_t, term_t,
+  // etc. aren't "smart" in the same sense, but the objects they refer
+  // to are garbage collected by Prolog and some care is needed to
   // ensure they have appropriate reference counts (e.g.,
   // PlAtom::register_ref() and PlTerm::record()).
-  void reset(C_t v = null) { C_ = v; }
+  void reset() { C_ = null; } // same as set_null()
+  void reset(WrappedC<C_t> v) { C_ = v.C_; }
 };
 
 // TODO: use PlEncoding wherever a method takes a char* or std::string.
@@ -220,6 +222,39 @@ public:
   }
 };
 
+		 /*******************************
+		 *  PL_{acquire,release}_stream *
+		 *******************************/
+
+// TODO: document this.
+// In brief, this is RAII for PL_{acquire,release_stream}.
+// To use:
+//    PlAcquireStream strm(other_stream);
+//    Sfprintf(strm, ...);
+
+class PlAcquireStream
+{
+public:
+  explicit PlAcquireStream(IOSTREAM *s)
+    : s_(Plx_acquire_stream(s))
+  { PlCheckFail(s_ != nullptr);
+  }
+
+  operator IOSTREAM *()
+  { return s_;
+  }
+
+  // The following has an implicit throw of PlFail if
+  // PL_release_stream() detects an IO error had happened
+  ~PlAcquireStream()
+  { if ( s_ )
+    { Plx_release_stream(s_);
+   }
+  }
+
+private:
+  IOSTREAM *s_ = nullptr;
+};
 
 		 /*******************************
 		 *	 PROLOG CONSTANTS	*
@@ -274,6 +309,8 @@ public:
   bool operator !=(const wchar_t *s) const { return !eq(s); }
   [[deprecated("use PlAtom instead of atom_t")]] bool operator !=(atom_t to) const { return C_ != to; }
 
+  // TODO: when C++17 becomes standard, rename register_ref() to register().
+
   void register_ref() const
   { Plx_register_atom(C_);
   }
@@ -382,6 +419,16 @@ public:
   { Plx_put_atom(C_, a.C_);
   }
 
+  // The following constructor is the same as to PlTerm_term_t(); the
+  // latter is for consistency with other constructors
+  // (PlTerm_integer(), etc.)  and the former is to make some template
+  // programming eaiser.
+  explicit PlTerm(term_t t)
+    : WrappedC<term_t>(t)
+  { }
+
+  explicit PlTerm(const PlRecord& r);
+
   PlTerm(const PlTerm&) = default;
 
   // TODO: PlTerm& operator =(const PlTerm&) = delete; // TODO: when the deprecated items below are removed
@@ -430,9 +477,9 @@ public:
   void get_char_ex(int *p, int eof) const { Plx_get_char_ex(C_, p, eof); }
   void unify_bool_ex(int val)       const { Plx_unify_bool_ex(C_, val); }
   void get_pointer_ex(void **addrp) const { Plx_get_pointer_ex(C_, addrp); }
-  void unify_list_ex(PlTerm h, PlTerm t) const { Plx_unify_list_ex(C_, h.C_, t.C_); }
+  bool unify_list_ex(PlTerm h, PlTerm t) const { return Plx_unify_list_ex(C_, h.C_, t.C_); }
   void unify_nil_ex()               const { Plx_unify_nil_ex(C_); }
-  void get_list_ex(PlTerm h, PlTerm t) const { Plx_get_list_ex(C_, h.C_, t.C_); }
+  bool get_list_ex(PlTerm h, PlTerm t) const { return Plx_get_list_ex(C_, h.C_, t.C_); }
   void get_nil_ex()                 const {  Plx_get_nil_ex(C_); }
 
   int type()         const { return Plx_term_type(C_); } // PL_VARIABLE, PL_ATOM, etc.
@@ -600,6 +647,7 @@ public:
 
 
   [[nodiscard]] bool unify_blob(const PlBlob* blob) const;
+  [[nodiscard]] bool unify_blob(std::unique_ptr<PlBlob>* blob) const;
   [[nodiscard]] bool unify_blob(const void *blob, size_t len, const PL_blob_t *type) const
   { return Plx_unify_blob(C_, const_cast<void*>(blob), len, const_cast<PL_blob_t*>(type)); }
 
@@ -644,11 +692,6 @@ public:
 
   void reset_term_refs() { Plx_reset_term_refs(C_); }
 
-protected:
-  explicit PlTerm(term_t t) // See PlTerm_term_t for a better constructor
-    : WrappedC<term_t>(t)
-  { }
-
 private:
   bool eq(const char *s) const;
   bool eq(const wchar_t *s) const;
@@ -687,6 +730,7 @@ public:
 class PlTerm_term_t : public PlTerm
 {
 public:
+  // TODO: [[deprecated("use PlTerm(Plterm::null)")]]
   explicit PlTerm_term_t(term_t t)
     : PlTerm(t) {}
 };
@@ -1007,7 +1051,7 @@ public:
 
   // plThrow() is for the try-catch in PREDICATE - returns the result
   // of Plx_raise_exception(), which is always `false`, as a foreign_t.
-  virtual foreign_t plThrow()
+  virtual foreign_t plThrow() const
   { foreign_t rc = static_cast<foreign_t>(Plx_raise_exception(term().C_));
     return rc;
   }
@@ -1018,7 +1062,7 @@ public:
 
 protected:
   explicit PlException(term_t ex)
-    : term_rec_(PlTerm_term_t(ex)) { }
+    : term_rec_(PlTerm(ex)) { }
 
   void set_what_str()
   { if ( what_str_.empty() )
@@ -1043,9 +1087,9 @@ protected:
 
 PlException PlGeneralError(PlTerm inside);
 
-PlException PlTypeError(const char *expected, const PlTerm& actual);
+PlException PlTypeError(const std::string& expected, const PlTerm& actual);
 
-PlException PlDomainError(const char *expected, const PlTerm& actual);
+PlException PlDomainError(const std::string& expected, const PlTerm& actual);
 
 PlException PlDomainError(const PlTerm& expected, const PlTerm& actual);
 
@@ -1053,31 +1097,31 @@ PlException PlInstantiationError(const PlTerm& t);
 
 PlException PlUninstantiationError(const PlTerm& t);
 
-PlException PlRepresentationError(const char *resource);
+PlException PlRepresentationError(const std::string& resource);
 
-PlException PlExistenceError(const char *type, PlTerm actual);
+PlException PlExistenceError(const std::string& type, PlTerm actual);
 
-PlException PlPermissionError(const char *op, const char *type, const PlTerm& obj);
+PlException PlPermissionError(const std::string& op, const std::string& type, const PlTerm& obj);
 
-PlException PlResourceError(const char *resource);
+PlException PlResourceError(const std::string& resource);
 
-PlException PlUnknownError(const char *description);
+PlException PlUnknownError(const std::string& description);
 
-void PlWrap_impl(qid_t qid);
+void PlWrap_fail(qid_t qid);
 
 template<typename C_t> C_t
 PlWrap(C_t rc, qid_t qid)
 { if ( rc == static_cast<C_t>(0) )
-    PlWrap_impl(qid);
+    PlWrap_fail(qid);
   return rc;
 }
 
-void PlEx_impl(qid_t qid);
+void PlEx_fail(qid_t qid);
 
 template<typename C_t> void
 PlEx(C_t rc, qid_t qid)
 { if ( rc == static_cast<C_t>(0) )
-    PlEx_impl(qid);
+    PlEx_fail(qid);
 }
 
 
@@ -1126,6 +1170,14 @@ public:
   [[nodiscard]] void *foreign_context_address() const { return Plx_foreign_context_address(C_); }
 
   [[nodiscard]] PlPredicate foreign_context_predicate() const { return PlPredicate(Plx_foreign_context_predicate(C_)); }
+
+  template <typename ContextType>
+  [[nodiscard]]
+  std::unique_ptr<ContextType>
+  context_unique_ptr()
+  { return std::unique_ptr<ContextType>(static_cast<ContextType *>(foreign_context_address()));
+  }
+
 };
 
 
@@ -1234,16 +1286,17 @@ public:
   // The return code from next_solution can be (if called with PL_Q_EXT_STATUS):
   //    TRUE
   //    FALSE
+  //    PL_S_NOT_INNER
   //    PL_S_EXCEPTION
   //    PL_S_FALSE
   //    PL_S_TRUE
   //    PL_S_LAST
-  // Because of this, you shouldn't use PlCheck(q.next_solution())
+  // Because of this, you shouldn't use PlCheckFail(q.next_solution())
   [[nodiscard]] int next_solution();
 
   PlTerm exception() const
   { verify(); // Not meaningful if cut() or close_destroy() has been done
-    return PlTerm_term_t(Plx_exception(exception_qid()));
+    return PlTerm(Plx_exception(exception_qid()));
   }
 
   qid_t exception_qid() const
@@ -1251,7 +1304,7 @@ public:
   }
 
   PlTerm yielded() const
-  { return PlTerm_term_t(Plx_yielded(C_));
+  { return PlTerm(Plx_yielded(C_));
   }
 
   void cut()
@@ -1390,24 +1443,34 @@ public:
 #define PROLOG_MODULE static_cast<const char*>(nullptr)
 #endif
 
+// This macro is used by both the PREDICATE macros and the blob callbacks
+
+#define PREDICATE_CATCH(error_action) \
+	    catch ( const std::bad_alloc& ) \
+	  { (void)Plx_resource_error("memory"); \
+            error_action; \
+	  } catch ( const PlFail& ) \
+	  { error_action; \
+	  } catch ( const PlExceptionFail& ) \
+          { error_action; \
+	  } catch ( const PlException& ex ) \
+	  { ex.plThrow(); \
+            error_action; \
+	  }
+
 #define NAMED_PREDICATE(plname, name, arity) \
 	static foreign_t \
 	pl_ ## name ## __ ## arity(PlTermv PL_av); \
 	static foreign_t \
 	_pl_ ## name ## __ ## arity(term_t t0, int a, control_t c) \
 	{ (void)a; (void)c; \
+          foreign_t rc; \
 	  try \
 	  { \
-	    return pl_ ## name ## __ ## arity(PlTermv(arity, PlTerm_term_t(t0))); \
-	  } catch ( const std::bad_alloc& ) \
-	  { return static_cast<foreign_t>(Plx_resource_error("memory")); \
-	  } catch ( const PlFail& ) \
-          { return false;     \
-	  } catch ( const PlExceptionFail& ) \
-          { return false;     \
-	  } catch ( PlException& ex ) \
-          { return ex.plThrow();  \
+	    rc = pl_ ## name ## __ ## arity(PlTermv(arity, PlTerm(t0))); \
 	  } \
+          PREDICATE_CATCH(rc = false) \
+          return rc; \
 	} \
 	static PlRegister _x ## name ## __ ## arity(PROLOG_MODULE, plname, arity, \
 					    _pl_ ## name ## __ ## arity); \
@@ -1419,18 +1482,13 @@ public:
 	static foreign_t \
 	_pl_ ## name ## __0(term_t t0, int a, control_t c) \
 	{ (void)t0; (void)a; (void)c; \
+          foreign_t rc; \
 	  try \
 	  { \
-	    return pl_ ## name ## __0(); \
-	  } catch ( std::bad_alloc& ) \
-	  { return static_cast<foreign_t>(Plx_resource_error("memory")); \
-	  } catch ( PlFail& ) \
-	  { return false; \
-	  } catch ( const PlExceptionFail& ) \
-          { return false;     \
-	  } catch ( PlException& ex ) \
-	  { return ex.plThrow(); \
+	    rc = pl_ ## name ## __0(); \
 	  } \
+          PREDICATE_CATCH(rc = false) \
+          return rc; \
 	} \
 	static PlRegister _x ## name ## __0(PROLOG_MODULE, plname, 0, \
 					    _pl_ ## name ## __0); \
@@ -1442,19 +1500,14 @@ public:
 	static foreign_t \
 	_pl_ ## name ## __ ## arity(term_t t0, int a, control_t c) \
 	{ (void)a; \
+          foreign_t rc; \
 	  try \
 	  { \
 	    /* t0.C_ is 0 if handle.foreign_control()==PL_PRUNED */ \
-	    return pl_ ## name ## __ ## arity(PlTermv(arity, PlTerm_term_t(t0)), PlControl(c)); \
-	  } catch ( std::bad_alloc& ) \
-	  { return static_cast<foreign_t>(Plx_resource_error("memory")); \
-	  } catch ( PlFail& ) \
-	  { return false; \
-	  } catch ( const PlExceptionFail& ) \
-          { return false;     \
-	  } catch ( PlException& ex ) \
-	  { return ex.plThrow(); \
+	    rc = pl_ ## name ## __ ## arity(PlTermv(arity, PlTerm(t0)), PlControl(c)); \
 	  } \
+          PREDICATE_CATCH(rc = false) \
+          return rc; \
 	} \
 	static PlRegister _x ## name ## __ ## arity(PROLOG_MODULE, plname, arity, \
 						    _pl_ ## name ## __ ## arity, \
@@ -1489,62 +1542,52 @@ public:
 #define A10	PL_A10
 #endif
 
+// TODO: delete this section
 		 /*******************************
 		 *     NONDET HELPERS		*
 		 *******************************/
 
-// For non-deterministic predicates that allocate a context, the
-// PlForeignContextPtr is a RAII (Resource Acquisition Is
-// Initialization) class (a kind of "smart pointer") that holds a
-// pointer that is deleted at exit (whether by return or exception)
-// unless deferred_free_ is false. This simplifies code that can throw
-// exceptions such as the PlTerm::as_long().
-//
-// The pointer must have been allocated using the `new` operator.  The
-// pointer can have the value nullptr.
-//
-// THe methods are the usual "smart pointer" ones: dereference (either
-// using `->` or `*`), get(), set(ptr). In addition:
-//    deferred_free() - the pointer will be deleted on return/throw (default)
-//    keep() - the pointer will not be deleted on return/throw
-// Typically, keep() is used only when calling PL_retry_address()
-
-template <typename ContextType> class PlForeignContextPtr
+template <typename ContextType>
+// TODO: [[deprecated("Use PlControl::context_unique_ptr")]]
+class PlForeignContextPtr
 {
 public:
+  [[deprecated("Use PlControl::context_unique_ptr")]]
   explicit PlForeignContextPtr<ContextType>(PlControl handle)
-    : ptr_(static_cast<ContextType *>(handle.foreign_context_address())),
-      deferred_free_(true)
-  { }
+  { ptr_.reset(static_cast<ContextType *>(handle.foreign_context_address()));
+  }
 
   PlForeignContextPtr<ContextType>(const PlForeignContextPtr<ContextType>&) = delete;
   PlForeignContextPtr<ContextType>(PlForeignContextPtr<ContextType>&&) = delete;
   PlForeignContextPtr<ContextType>& operator =(const PlForeignContextPtr<ContextType>&) = delete;
   PlForeignContextPtr<ContextType>& operator =(PlForeignContextPtr<ContextType>&&) = delete;
 
-  ContextType& operator*()  const { return *ptr_; }
-  ContextType* operator->() const { return ptr_; }
-  ContextType* get()        const { return ptr_; }
-  void set(ContextType* ptr = nullptr) { ptr_ = ptr; }
-
-  void deferred_free(bool v) { deferred_free_ = v; }
-  void deferred_free()       { deferred_free(true); }
-  ContextType* keep()        { deferred_free(false); return get(); }
-
-  ~PlForeignContextPtr()
-  { if ( deferred_free_ )
-      delete ptr_; // it's safe to delete nullptr
-  }
+  [[deprecated("Use PlControl::context_unique_ptr")]] ContextType& operator*()  const { return *ptr_; }
+  [[deprecated("Use PlControl::context_unique_ptr")]] ContextType* operator->() const { return ptr_.get(); }
+  [[deprecated("Use PlControl::context_unique_ptr")]] ContextType* get()        const { return ptr_.get(); }
+  [[deprecated("Use PlControl::context_unique_ptr")]] void set(ContextType* ptr = nullptr)   { reset(ptr); } // TODO: delete
+  [[deprecated("Use PlControl::context_unique_ptr")]] void reset(ContextType* ptr = nullptr) { ptr_.reset(ptr); }
+  [[deprecated("Use PlControl::context_unique_ptr")]] ContextType* keep()        { return release(); } // TODO: delete
+  [[deprecated("Use PlControl::context_unique_ptr")]] ContextType* release()     { return ptr_.release(); }
 
 private:
-  ContextType *ptr_;
-  bool deferred_free_;
+  std::unique_ptr<ContextType> ptr_;
 };
 
+
 		 /*******************************
 		 *          BLOBS		*
 		 *******************************/
 
+// PlBlobV is used by PL_BLOB_DEFINITION to create a vector of
+// C-callable functions that in turn call the methods inside PlBlob.
+// Although PlBlobV is defined as a class, it is really a namespace,
+// with a template variable C_t for convenience.
+//
+// TODO: This code assumes that the only difference between C and C++
+//       functions is the name mangling. If there are additional
+//       differences, this template class will need to be replaced by
+//       some macros that define a set of `extern "C"` functions.
 
 template<typename C_t>
 class PlBlobV
@@ -1556,6 +1599,8 @@ public:
   { size_t len;
     PL_blob_t *type;
     auto ref = static_cast<C_t *>(aref.blob_data(&len, &type));
+    // Can't throw PlException here because might be in a context
+    // outside of a PREDICATE.
     if ( ref && type == ref->blob_t_ )
     { assert(len == sizeof *ref);
       return ref;
@@ -1567,59 +1612,116 @@ public:
   static C_t*
   cast_check(PlAtom aref)
   { auto ref = cast(aref);
+    // Can't throw PlException here because might be in a context
+    // outside of a PREDICATE.
     assert(ref);
     return ref;
   }
 
+  [[nodiscard]]
+  static C_t*
+  cast_ex(PlTerm t, const PL_blob_t& b)
+  { auto ref = cast(t.as_atom());
+    if ( !ref )
+      throw PlTypeError(b.name, t);
+    return ref;
+  }
+
   static void acquire(atom_t a)
   { PlAtom a_(a);
     auto data = cast_check(a_);
-    data->acquire(a_);
+    bool rc;
+    try
+    { data->acquire(a_);
+      rc = true;
+    }
+    PREDICATE_CATCH(rc = false)
+      // TODO: if ( ! rc ) Plx_clear_exception() ?
+    assert(rc);
+    (void)rc;
   }
 
   [[nodiscard]]
-  static int release(atom_t a)
+  static int release(atom_t a) noexcept
   { auto data = cast_check(PlAtom(a));
-    data->release2();
     delete data;
     return true;
   }
 
   [[nodiscard]]
   static int compare(atom_t a, atom_t b)
-  { const auto a_data = cast_check(PlAtom(a)); // TODO: cast(...)
-    const auto b_data = cast_check(PlAtom(b));
-    int rc = a_data->compare_fields(b_data);
-    if ( rc == 0 )
-    { return (a_data < b_data) ? -1 : (a_data > b_data) ? 1 : 0;
+  { if (a == b) // Just in case Prolog didn't check this.
+      return 0;
+    // Prolog should never give us two atoms (blobs) with different
+    // types - they should have been already compared by standard
+    // order of types; but use cast_check() anyway (which will be
+    // optimised away if NDEBUG).
+    bool rc_try;
+    int rc = -1; // Stop the compiler warning about uninitialized
+    try
+    { const auto a_data = cast_check(PlAtom(a)); // TODO: cast(PlAtom(a))
+      const auto b_data = cast_check(PlAtom(b));
+      rc = a_data->compare_fields(b_data);
+      if ( rc == 0 )
+      { rc = (a_data < b_data) ? -1 : (a_data > b_data) ? 1 : 0;
+      }
+      rc_try = true;
     }
+    PREDICATE_CATCH(rc_try = false)
+      // TODO: if ( ! rc_try ) Plx_clear_exception() ?
+    assert(rc_try);
+    (void)rc_try;
     return rc;
   }
 
   [[nodiscard]]
   static int write(IOSTREAM *s, atom_t a, int flags)
   { const auto data = cast_check(PlAtom(a));
-    return data->write(s, flags);
+    int rc;
+    try
+    { rc = data->write(s, flags);
+    }
+    PREDICATE_CATCH(rc = -1)
+      // TODO: if ( rc < 0 ) Plx_clear_exception() ?
+    return rc;
   }
 
   [[nodiscard]]
   static int save(atom_t a, IOSTREAM *fd)
   { const auto data = cast_check(PlAtom(a));
-    return data->save(fd);
+    bool rc;
+    try
+    { data->save(fd);
+      rc = true;
+    }
+    PREDICATE_CATCH(rc = false)
+      // TODO: if ( ! rc ) Plx_clear_exception() ?
+    return rc;
   }
 
   [[nodiscard]]
   static atom_t load(IOSTREAM *fd)
   { C_t ref;
-    return ref.load(fd).C_;
+    atom_t atom;
+    int rc_try;
+    try
+    { atom = ref.load(fd).C_;
+      rc_try = true;
+    }
+    PREDICATE_CATCH(rc_try = false);
+    if ( !rc_try )
+    { // TODO: Plx_clear_exception() ?
+      return PlAtom::null;
+    }
+    return atom;
   }
 };
 
 
 #define PL_BLOB_DEFINITION(blob_class, blob_name) \
-{ .magic   = PL_BLOB_MAGIC,		\
-  .flags   = PL_BLOB_NOCOPY,		\
-  .name    = blob_name,			\
+{ .magic   = PL_BLOB_MAGIC,			\
+  .flags   = PL_BLOB_NOCOPY,			\
+  .name    = blob_name,				\
   .release = PlBlobV<blob_class>::release,	\
   .compare = PlBlobV<blob_class>::compare,	\
   .write   = PlBlobV<blob_class>::write,	\
@@ -1644,21 +1746,18 @@ public:
 
   virtual size_t blob_size_() const = 0; // See PL_BLOB_SIZE
 
-  void acquire(PlAtom _symbol) {
-    symbol_ = _symbol;
-    acquire2();
+  void acquire(PlAtom _symbol)
+  { symbol_ = _symbol;
+    // Don't: symbol_.register_ref() because it's already got a
+    // reference from when the blob was allocated, (by
+    // PL_unify_blob(), PL_put_blob(), or PL_new_blob()); extra
+    // reference count would prevent its GC.
   }
 
-  virtual void acquire2() { }
-
-  virtual void release2() { }
-
-  bool symbol_not_null() const { return symbol_.not_null(); }
-
   PlTerm symbol_term() const;
 
   virtual int compare_fields(const PlBlob *_b) const
-  { return 0;
+  { return 0; // compare() will do bitwise comparison
   }
 
   bool write(IOSTREAM *s, int flags) const;
@@ -1666,52 +1765,16 @@ public:
   bool virtual write_fields(IOSTREAM *s, int flags) const
   { return true; }
 
-  virtual bool save(IOSTREAM *fd) const;
+  virtual void save(IOSTREAM *fd) const;
 
   virtual PlAtom load(IOSTREAM *fd);
 
   const PL_blob_t* blob_t_ = nullptr;
 
-  // associated symbol (used for error terms)
-  // filed in by acquire():
+  // Associated symbol (used for error terms) filed in by acquire()
+  // and usually accessed by the symbol_term() method
   PlAtom symbol_ = PlAtom(PlAtom::null);
 };
 
 
-inline
-bool PlBlob::write(IOSTREAM *s, int flags) const
-{ return Sfprintf(s, "<%s>(%p", blob_t_->name, this) &&
-    write_fields(s, flags) &&
-    // In general, the flags are handled by the calling
-    // Plx_write_term(), so don't check for, e.g., flags&PL_WRT_NEWLINE.
-    Sfprintf(s, ")");
-}
-
-inline
-bool PlBlob::save(IOSTREAM *fd) const
-{ return PL_warning("Cannot save reference to <%s>(%p)", blob_t_->name, this);
-}
-
-inline
-PlAtom PlBlob::load(IOSTREAM *fd)
-{ (void)PL_warning("Cannot load reference to <%s>", blob_t_->name);
-  PL_fatal_error("Cannot load reference to <%s>", blob_t_->name);
-  return PlAtom(PlAtom::null);
-}
-
-inline
-PlTerm PlBlob::symbol_term() const
-{ if ( symbol_not_null() )
-    return PlTerm_atom(symbol_);
-  return PlTerm_var();
-}
-
-inline
-bool PlTerm::unify_blob(const PlBlob* blob) const
-{ return PlTerm::unify_blob(static_cast<const void*>(blob),
-                            blob->blob_size_(), blob->blob_t_);
-}
-
-
-
 #endif /*_SWI_CPP2_H*/
