@@ -1187,18 +1187,20 @@ layout(h4,         2-0, 0-2).
 
 layout(iframe,     1-1, 1-1).
 
-layout(hr,         1-1, empty).         % empty elements
-layout(br,         0-1, empty).
-layout(img,        0-0, empty).
-layout(meta,       1-1, empty).
-layout(base,       1-1, empty).
-layout(link,       1-1, empty).
-layout(input,      0-0, empty).
-layout(frame,      1-1, empty).
-layout(col,        0-0, empty).
 layout(area,       1-0, empty).
+layout(base,       1-1, empty).
+layout(br,         0-1, empty).
+layout(col,        0-0, empty).
+layout(embed,      1-1, empty).
+layout(hr,         1-1, empty).         % empty elements
+layout(img,        0-0, empty).
 layout(input,      1-0, empty).
+layout(link,       1-1, empty).
+layout(meta,       1-1, empty).
 layout(param,      1-0, empty).
+layout(source,     1-0, empty).
+layout(track,	   1-0, empty).
+layout(wbr,	   0-0, empty).
 
 layout(p,          2-1, -).             % omited close
 layout(td,         0-0, 0-0).
