@@ -193,7 +193,8 @@ Title = 'Free Online Version - Learn Prolog
                        headers(-list),
                        raw_headers(-list(string)),
                        connection(+atom),
-                       method(oneof([delete,get,put,head,post,patch,options])),
+                       method(oneof([delete,get,put,purge,head,
+                                     post,patch,options])),
                        size(-integer),
                        status_code(-integer),
                        output(-stream),
