@@ -234,7 +234,7 @@ updateAlerted(PL_local_data_t *ld)
 #endif
   if ( ld->fli.string_buffers.top )		mask |= ALERT_BUFFER;
   if ( UNDO_SCHEDULED(ld) )			mask |= ALERT_UNDO;
-  if ( LD->coverage.active )			mask |= ALERT_COVERAGE;
+  if ( ld->coverage.active )			mask |= ALERT_COVERAGE;
 
   ld->alerted = mask;
 }
@@ -1431,7 +1431,9 @@ __do_undo(DECL_LD mark *m)
   while(--tt >= mt)
   { Word p = tt->address;
 
-    if ( isTrailVal(p) )
+    if ( likely(!isTrailVal(p)) )
+    { setVar(*p);
+    } else
     { DEBUG(2, Sdprintf("Undoing a trailed assignment\n"));
       tt--;
       if ( tt->address == gBase )
@@ -1440,9 +1442,8 @@ __do_undo(DECL_LD mark *m)
       DEBUG(CHK_SECURE,
 	    if ( isAttVar(*tt->address) )
 	      assert(on_attvar_chain(tt->address)));
-      assert(!(*tt->address & (MARK_MASK|FIRST_MASK)));
-    } else
-      setVar(*p);
+      DEBUG(0, assert(!(*tt->address & (MARK_MASK|FIRST_MASK))));
+    }
   }
 
   DEBUG(CHK_SECURE,
@@ -1452,15 +1453,8 @@ __do_undo(DECL_LD mark *m)
 
   tTop = mt;
 
-  Word ngtop;
-  if ( LD->frozen_bar > m->globaltop )
-  { DEBUG(CHK_SECURE, assert(gTop >= LD->frozen_bar));
-    reclaim_attvars(LD->frozen_bar);
-    ngtop = LD->frozen_bar;
-  } else
-  { reclaim_attvars(m->globaltop);
-    ngtop = m->globaltop;
-  }
+  Word ngtop = max(LD->frozen_bar, m->globaltop);
+  reclaim_attvars(ngtop);
 
   DEBUG(CHK_SECURE,
 	{ for(Word p = gTop; --p > ngtop;)
@@ -3060,7 +3054,7 @@ typedef struct register_file
 #define VMH_ARGSTRUCT(Name)		struct helper_args_ ## Name
 
 /* GCC and CLang allow for struct name {}, i.e., an empty struct */
-#if O_EMPY_STRUCTS
+#if O_EMPTY_STRUCTS
 #define VMH_PAD_STRUCT
 #define VMH_INIT_ARGSTRUCT(...) {__VA_ARGS__}
 #else
@@ -3123,7 +3117,8 @@ static vmi_instr jmp_table[] =
 #undef NEXT_INSTRUCTION
 #undef VMH_GOTO
 #define NEXT_INSTRUCTION (void)0
-#define VMH_GOTO(n) PC = helper_##n(VMI_ARG_PASS, (VMH_ARGSTRUCT(n)){})
+#define VMH_GOTO(n) PC = helper_##n(VMI_ARG_PASS, \
+				    (VMH_ARGSTRUCT(n))VMH_INIT_ARGSTRUCT())
 
 #if VMI_USE_REGISTER_VARIABLES
 # undef LD
@@ -3182,7 +3177,8 @@ API_STUB(int)
 int
 PL_next_solution(DECL_LD qid_t qid)
 { register_file REGISTERS =
-  { .qid = qid
+  { .qid = qid,
+    .fndet_context.engine = LD
 #if VMCODE_IS_ADDRESS
     , .nop1 = 0, .nop2 = 0
 #endif
